//------------------------------------------------------------------------------
// CERN BE-CO-HT
// Mock Turtle
// https://gitlab.cern.ch/coht/mockturtle
//------------------------------------------------------------------------------
//
// unit name:   main
//
// description: A SystemVerilog testbench to test the ethernet endpoint of MT
//
//------------------------------------------------------------------------------
// Copyright CERN 2018
//------------------------------------------------------------------------------
// Copyright and related rights are licensed under the Solderpad Hardware
// License, Version 2.0 (the "License"); you may not use this file except
// in compliance with the License. You may obtain a copy of the License at
// http://solderpad.org/licenses/SHL-2.0.
// Unless required by applicable law or agreed to in writing, software,
// hardware and materials distributed under this License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
// or implied. See the License for the specific language governing permissions
// and limitations under the License.
//------------------------------------------------------------------------------

`include "mock_turtle_driver.svh"
`include "vhd_wishbone_master.svh"


import mock_turtle_pkg::*;
import wr_fabric_pkg::*;
import mt_mqueue_pkg::*;


`include "stream_mon.svh"
`include "string_utils.svh"

`timescale 1ns/1ps

typedef byte byte_vector_t[$];

function automatic byte_vector_t loadPacketFromFile(string name);
   integer fd;
   byte_vector_t rv;



   fd = $fopen(name, "r");

   if (fd == 0)
     $error("Can't open '%s'", name);


   while(1)
     begin
	string s;
	int    n, i;
	string_array_t tokens;


	n = $fgets(s, fd);

	if(n<=0)
	  break;

	tokens = tokenizeString(s);

	for(i=1; i < tokens.size(); i++)
	  rv.push_back( parseHex(tokens[i] ) );


     end

   return rv;

endfunction // loadPacketFromFile


module main;

   reg rst_n = 0;
   reg clk_sys = 0;

   t_mt_rmq_endpoint_iface_out mt2ep;
   t_mt_rmq_endpoint_iface_in ep2mt;
   t_wrf_source_out eth_src_out;
   t_wrf_source_in eth_src_in;
   t_wrf_sink_out eth_snk_out;
   t_wrf_sink_in eth_snk_in;

   always #8ns clk_sys <= ~clk_sys;

   initial begin
      repeat(20) @(posedge clk_sys);
      rst_n = 1;
   end

//`undef USE_INTEGRATED_ENDPOINT

   mock_turtle_core #
     (
      )
   DUT (
	.clk_i         (clk_sys),
	.rst_n_i       (rst_n),
	.host_slave_i  (Host.out),
	.host_slave_o  (Host.in),
	.sp_master_o   (),
	.sp_master_i   (),
	.dp_master_o   (),
	.dp_master_i   (),

	.rmq_endpoint_o(mt2ep),
	.rmq_endpoint_i(ep2mt),

	.clk_ref_i     (),
	.tm_i          (),
	.gpio_o        (),
	.gpio_i        (),
	.hmq_in_irq_o  (),
	.hmq_out_irq_o (),
	.notify_irq_o  (),
	.console_irq_o ()
	);

   mt_rmq_ethernet_endpoint
     EP (
	 .clk_i  (clk_sys),
	 .rst_n_i(rst_n),

	 .mt_rmq_i(mt2ep),
	 .mt_rmq_o(ep2mt),


	 .eth_src_o (eth_src_out),
	 .eth_src_i (eth_src_in),

	 .eth_snk_o (eth_snk_out),
	 .eth_snk_i (eth_snk_in)
	 );

   assign eth_src_in.ack = 1'b1;
   assign eth_src_in.stall = 1'b0;
   assign eth_src_in.err = 1'b0;
   assign eth_src_in.rty = 1'b0;


//   assign eth_snk_in.cyc = 1'b0;

/*   stream_mon MonOut
     (
      .clk_i(clk_sys),
      .rst_n_i(rst_n),

      .snk_in_i(rmq_src_out[0][0]),
      .snk_out_i(rmq_src_in[0][0])
      );*/

   wr_fabric_master EthSrc(
      .clk_i(clk_sys),
      .rst_n_i(rst_n),

      .src_i(eth_snk_out),
      .src_o(eth_snk_in)
   );


   IVHDWishboneMaster Host ( clk_sys, rst_n );

   IMockTurtleIRQ IrqMonitor (`MT_ATTACH_IRQ(DUT));

   string fw = "../../../tests/firmware/rmq-udp-send/fw-rmq-udp-send.bin";

   const uint64_t mt_base = 'h0000;

   MockTurtleDriver drv;

   initial begin
      automatic byte_vector_t pkt =       loadPacketFromFile("1.dump");

      #10us;

      drv = new (Host.get_accessor(), mt_base, IrqMonitor, 1'b0);

      drv.init();

      drv.enable_console_irq (0, 1);

      drv.enable_hmqi_irq (0, 0, 1);

      drv.load_firmware (0, fw, 1'b0);

      drv.reset_core (0, 0);

      fork

	 forever begin
	    drv.update ();
	    # 1us;
	 end

	 begin
	    #100us;
	    EthSrc.send(pkt);

	 end

      join


   end // initial begin

endmodule // main
